@@ -874,7 +874,7 @@ class Nth(RqlQuery):
     def compose(self, args, optargs):
         return T(args[0], '[', args[1], ']')
 
-class Match(RqlQuery):
+class Match(RqlMethodQuery):
     tt = p.Term.MATCH
     st = 'match'
 
@@ -1137,11 +1137,12 @@ class Func(RqlQuery):
         vrs = []
         vrids = []
         for i in xrange(lmbd.func_code.co_argcount):
-            vrs.append(Var(Func.nextVarId))
-            vrids.append(Func.nextVarId)
             Func.lock.acquire()
+            var_id = Func.nextVarId
             Func.nextVarId += 1
             Func.lock.release()
+            vrs.append(Var(var_id))
+            vrids.append(var_id)
 
         self.vrs = vrs
         self.args = [MakeArray(*vrids), expr(lmbd(*vrs))]
